/* -*- mode: c++; c-basic-offset: 3; -*- */
#include "format_spec.hh"
#include <sstream>
#include <iomanip>
#include <stdexcept>
#include <cstdlib>

using namespace table;
using namespace std;

//======================================  Constructor
format_spec::format_spec(void)
   : mNumFormat(kDefault), mPrecise(0), mWidth(0)
{}

//======================================  Constructor
format_spec::format_spec(const std::string& fmt)
   : mNumFormat(kDefault), mPrecise(0), mWidth(0)
{
   if (!fmt.empty()) parse(fmt);
}

//======================================  Destructor
format_spec::~format_spec(void) {
}

//======================================  Parse a format string
void
format_spec::parse(const std::string& fmt) {
   string::size_type len = fmt.size();
   for (string::size_type i=0; i<len; ) {
      char fmtch = fmt[i++];
      const char* pin  = fmt.c_str() + i;
      char*       pout = const_cast<char*>(pin);
      switch (fmtch) {
      case 'f':
	 setFixed(strtol(pin, &pout, 0));
	 break;
      case 'p':
	 setPrecision(strtol(pin, &pout, 0));
	 break;
      case 's':
	 setScientific(strtol(pin, &pout, 0));
	 break;
      case 'w':
	 mWidth = strtol(pin, &pout, 0);
	 break;
      default:
	 throw runtime_error(string("Unknown format: ") + fmtch);
      }
      i += pout - pin;
   }
}

//======================================  center a string;
inline std::string
center(std::string rslt) {
   size_t nspc = rslt.find_first_not_of(" ");
   if (nspc != string::npos && nspc/2 != 0) {
      rslt += rslt.substr(0, nspc/2);
      rslt.erase(0, nspc/2);
   }
   return rslt;
}

//======================================  Put a string
std::string 
format_spec::put_string(double num) const {
   ostringstream ostr;
   if (mWidth) ostr << setw(mWidth);

   //------------------------------------  Set the justification
   switch (mJustify) {
   case kjLeft:
      ostr << left;
      break;
   case kjCenter:
   case kjRight:
      ostr << right;
      break;
   default:
      break;
   }

   //------------------------------------  Convert float data
   switch (mNumFormat) {

      //------------------------------------  C++ default formatting.
   case kDefault: {
      //--------------------------------  Check for integer data
      long inum = num;
      if (double(inum) == num) ostr << inum;
      else                     ostr << setprecision(mPrecise) << num;
      break;
   }

      //------------------------------------  Fixed point notation.
   case kFixed:
      ostr << fixed << setprecision(mPrecise) << num;
      break;

      //------------------------------------  Scientific notation
   case kSci:
      ostr << scientific << setprecision(mPrecise) << num;
      break;
   }

   //------------------------------------  Return the converted data
   if (mJustify == kjCenter) {
      return center(ostr.str());
   }
   return ostr.str();
}

//======================================  Put a string
std::string 
format_spec::put_string(const std::string& str) const {
   if (!mWidth) return str;

   ostringstream ostr;
   ostr << setw(mWidth);

   //------------------------------------  Set the justification
   switch (mJustify) {
   case kjLeft:
      ostr << left;
      break;
   case kjCenter:
   case kjRight:
      ostr << right;
      break;
   default:
      break;
   }

   ostr << str;

   //------------------------------------  Return the converted data
   if (mJustify == kjCenter) {
      return center(ostr.str());
   }
   return ostr.str();
}

//======================================  Set fixed point format
void 
format_spec::setFixed(int prec) {
   mNumFormat = kFixed;
   mPrecise   = prec;
}

//======================================  Set fixed point format
void 
format_spec::setJustify(const std::string& just) {
   if (just == "left") {
      mJustify = kjLeft;
   } else if (just == "right") {
      mJustify = kjRight;
   } else if (just == "center") {
      mJustify = kjCenter;
   } else {
      mJustify = kjNone;
   }
}

//======================================  Set default format
void 
format_spec::setPrecision(int prec) {
   mNumFormat = kDefault;
   mPrecise   = prec;
}

//======================================  Set Scientific notation
void 
format_spec::setScientific(int prec) {
   mNumFormat = kSci;
   mPrecise   = prec;
}

//======================================  Set Scientific notation
void 
format_spec::setWidth(int wid) {
   mWidth = wid;
}
